# -*- coding: utf-8 -*-
import click
import pip
import requests
from packaging.specifiers import SpecifierSet
from .errors import DatabaseFetchError
from .constants import DATABASE_MIRRORS, REQUEST_TIMEOUT
from collections import namedtuple


class Vulnerability(namedtuple("Vulnerability", ["name", "spec", "version", "data"])):

    @property
    def source(self):
        return self.cve_id if self.is_cve else "changelog"

    @property
    def is_cve(self):
        return "cve" in self.data

    @property
    def is_changelog(self):
        return "changelog" in self.data

    @property
    def cve_id(self):
        return self.data["cve"]

    @property
    def description(self):
        return self.data["description"] if self.is_cve else self.data["changelog"]


def fetch_database(full=False):
    for mirror in DATABASE_MIRRORS:
        db_name = "insecure_full.json" if full else "insecure.json"
        url = mirror + db_name
        r = requests.get(url=url, timeout=REQUEST_TIMEOUT)
        return r.json()
    raise DatabaseFetchError()


def get_vulnerabilities(pkg, spec, db):
    for entry in db[pkg]:
        if spec == entry["v"]:
            yield entry


def check(packages):
    db = fetch_database()
    db_full = None
    vulnerable_packages = frozenset(db.keys())
    vulnerable = []
    for pkg in packages:
        # normalize the package name, the safety-db is converting underscores to dashes and uses
        # lowercase
        name = pkg.key.replace("_", "-").lower()

        if name in vulnerable_packages:
            # we have a candidate here, build the spec set
            for specifier in db[name]:
                spec_set = SpecifierSet(specifiers=specifier)
                if spec_set.contains(pkg.version):
                    if not db_full:
                        db_full = fetch_database(full=True)
                    for data in get_vulnerabilities(pkg=name, spec=specifier, db=db_full):
                        vulnerable.append(
                            Vulnerability(name=name, spec=specifier, version=pkg.version, data=data)
                        )
    return vulnerable



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import os
import pip
import sys
import click
from safety import __version__
from safety import safety
from safety.formatter import report
from pkg_resources import parse_requirements
from collections import namedtuple

Package = namedtuple("Package", ["key", "version"])


def read_requirements(fh, resolve=False):
    """
    Reads requirements from a file like object and (optionally) from referenced files.
    :param fh: file like object to read from
    :param resolve: boolean. resolves referenced files.
    :return: generator
    """
    # filter out all "non-requirement" lines and resolve all referenced files.
    # "non-requirement" lines typically begin with a dash, e.g:
    # -e git://some-repo, or --index-server=https://foo.bar
    # lines referenced files start with a '-r'
    reqs = []
    for line in fh.readlines():
        if resolve and (line.startswith("-r") or line.startswith("--requirement")):
            # got a referenced file here, resolve it
            filename = line.strip("-r ").strip("--requirement").strip()
            # if there is a comment, remove it
            if " #" in filename:
                filename = filename.split(" #")[0].strip()
            basepath = os.path.dirname(fh.name)
            req_file = os.path.join(basepath, filename)
            if os.path.exists(req_file):
                # recursively yield the resolved requirements
                with open(req_file) as _fh:
                    for req in read_requirements(_fh, resolve=True):
                        yield req
        elif not line.startswith("-"):
            reqs.append(line)

    for req in parse_requirements("\n".join(reqs)):
        if len(req.specs) == 1 and req.specs[0][0] == "==":
            yield Package(key=req.key, version=req.specs[0][1])
        else:
            click.secho(
                "Warning: unpinned requirement '{req}' found, unable to check.".format(req=req.key),
                fg="yellow"
            )


@click.group()
@click.version_option(version=__version__)
def cli():
    pass


@cli.command()
@click.option("--full-report/--short-report", default=False)
@click.option("--stdin/--no-stdin", default=False)
@click.option("files", "--file", "-r", multiple=True, type=click.File())
def check(full_report, stdin, files):

    if files and stdin:
        click.secho("Can't read from --stdin and --file at the same time, exiting", fg="red")
        sys.exit(-1)

    if files:
        for f in files:
            packages = read_requirements(f, resolve=True)
    elif stdin:
        packages = read_requirements(sys.stdin)
    else:
        packages = pip.get_installed_distributions()

    vulns = safety.check(packages=packages)
    click.secho(report(vulns=vulns, full=full_report))
    sys.exit(-1 if vulns else 0)


if __name__ == "__main__":
    cli()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_safety
----------------------------------

Tests for `safety` module.
"""


import sys
import unittest
from contextlib import contextmanager
from click.testing import CliRunner

from safety import safety
from safety import cli



class TestSafety(unittest.TestCase):
    pass



# -*- coding: utf-8 -*-

DATABASE_MIRRORS = [
    "https://raw.githubusercontent.com/pyupio/safety-db/master/data/",
    # S3 mirrors
    "https://s3.eu-central-1.amazonaws.com/safety-db-mirror-1/data/",
    "https://s3-us-west-2.amazonaws.com/safety-db-mirror-2/data/",
    "https://s3.amazonaws.com/safety-db-mirror-3/data/"
]
REQUEST_TIMEOUT = 5


